@@ -1,19 +1,25 @@
+import json
+from os.path import dirname, isfile, join
 from uuid import uuid4
 
 from Bubot.Core.BubotHelper import BubotHelper
 from Bubot.Core.ObjForm import ObjForm
 from Bubot.Core.ObjModel import ObjModel
 from Bubot.Helpers.ActionDecorator import async_action
-from Bubot.Helpers.ExtException import KeyNotFound
+from Bubot.Helpers.ExtException import KeyNotFound, ExtException
 from Bubot.Helpers.Helper import Helper
+from Bubot.Helpers.Helper import get_tzinfo
 
 
 class Obj:
+    tzinfo = get_tzinfo()
     file = __file__  # должен быть в каждом файле наследники для чтения форм
     extension = False
+    is_subtype = False
     name = None
     key_property = '_id'
     uuid_id = True
+    _locales = {}
 
     def __init__(self, storage, *, account_id=None, lang=None, data=None, **kwargs):
         self.storage = storage
@@ -32,11 +38,14 @@ class Obj:
         )
 
     def init_by_data(self, data):
-        self.init()
-        if data:
-            Helper.update_dict(self.data, data)
-        if '_id' not in data:
-            self.data['_id'] = str(uuid4())
+        try:
+            self.init()
+            if data:
+                Helper.update_dict(self.data, data)
+            if '_id' not in data:
+                self.data['_id'] = str(uuid4())
+        except Exception as err:
+            raise ExtException(parent=err, action=f'{self.__class__.__name__}.init_by_data') from err
 
     # @classmethod
     # @async_action
@@ -61,7 +70,7 @@ class Obj:
         res = _action.add_stat(await self.find_one({"_id": _id}, _form=_form, **kwargs))
         if res:
             self.init_by_data(res)
-            return res
+            return self.data
         raise KeyNotFound(message=f'Object not found', detail=f'{self.obj_name} (id: {_id})')
 
     @async_action
@@ -101,6 +110,14 @@ class Obj:
     def obj_id(self, value):
         self.data['_id'] = value
 
+    @property
+    def uuid(self):
+        return self.data.get('_id')
+
+    @uuid.setter
+    def uuid(self, value):
+        self.data['_id'] = value
+
     @property
     def subtype(self):
         return self.data.get('subtype')
@@ -113,7 +130,8 @@ class Obj:
     async def list(self, *, _form="List", **kwargs):
         self.add_projection(_form, kwargs)
         kwargs = await self.list_set_default_params(**kwargs)
-        return await self.storage.list(self.db, self.obj_name, **kwargs)
+        result = await self.storage.list(self.db, self.obj_name, **kwargs)
+        return result
 
     async def list_set_default_params(self, **kwargs):
         return kwargs
@@ -129,7 +147,11 @@ class Obj:
         return await self.update(data, **kwargs)
 
     @async_action
-    async def update(self, data=None, **kwargs):
+    async def before_update(self, data=None, **kwargs):
+        pass
+
+    @async_action
+    async def update(self, data=None, _action=None, **kwargs):
         _data = data if data else self.data
         await self.set_default_params(_data)
         try:
@@ -137,6 +159,7 @@ class Obj:
         except KeyError:
             if self.uuid_id and not kwargs.get('where'):
                 _data['_id'] = str(uuid4())
+        _action.add_stat(await self.before_update(_data, **kwargs))
         res = await self.storage.update(self.db, self.obj_name, _data, **kwargs)
         if data is None:
             self.data = _data
@@ -161,8 +184,8 @@ class Obj:
 
     @async_action
     async def delete_many(self, where, *, _action=None):
-        await self.storage.delete_many(self.db, self.obj_name, where)
-        pass
+        result = await self.storage.delete_many(self.db, self.obj_name, where)
+        return result.raw_result
 
     @classmethod
     def get_form(cls, form_name):
@@ -231,3 +254,77 @@ class Obj:
             return self
         handler = BubotHelper.get_subtype_class(self.__class__.__name__, _subtype)
         return handler(self.storage, account_id=self.account_id, lang=self.lang, data=self.data)
+
+    @classmethod
+    def get_dir(cls):
+        return dirname(cls.file)
+
+    @classmethod
+    def read_i18n(cls, lang):
+        def i18n_path(cl):
+            return join(cl.get_dir(), 'i18n')
+
+        def _read_locale(cl, _locale, _locales):
+            locale_path = join(i18n_path(cl), f'{_locale}.json')
+            if isfile(locale_path):
+                with open(locale_path, "r", encoding='utf-8') as file:
+                    try:
+                        _data = json.load(file)
+                        if isinstance(_data, dict):
+                            try:
+                                _locales[_locale]
+                            except KeyError:
+                                _locales[_locale] = {}
+                            Helper.update_dict(_locales[_locale], _data)
+                        else:
+                            raise ExtException(
+                                message=f'Build locale',
+                                detail=f'{lang} for object {cl.__name__}: Bad format {_data}')
+                    except Exception as err:
+                        err = ExtException(parent=err)
+            return
+
+        try:
+            return cls._locales[lang]
+        except KeyError:
+            pass
+
+        if cls.is_subtype:
+            for elem in cls.__bases__:
+                if issubclass(elem, Obj):
+                    _read_locale(elem, lang, cls._locales)
+        _read_locale(cls, lang, cls._locales)
+        return cls._locales[lang]
+
+    @classmethod
+    def t(cls, value, lang):
+        locale = None
+        if lang:
+            locale = cls.read_i18n(lang)
+        if not locale and lang != 'en':
+            locale = cls.read_i18n('en')
+        return Helper.get_element_in_dict(locale, value, value)
+
+    def data_getter_root_dict(self, name):
+        try:
+            return self.data[name]
+        except KeyError:
+            self.data[name] = {}
+            return self.data[name]
+
+    def data_getter_root_str(self, name):
+        try:
+            return self.data[name]
+        except KeyError:
+            self.data[name] = ''
+            return self.data[name]
+
+    def data_getter_root_list(self, name):
+        try:
+            return self.data[name]
+        except KeyError:
+            self.data[name] = []
+            return self.data[name]
+
+    def data_setter_root(self, name, value):
+        self.data[name] = value
